!   E05SAF Example Program Text
!   Mark 23 Release. NAG Copyright 2011.

    MODULE e05safe_mod

!      E05SAF Example Program Module:
!             Parameters and User-defined Routines

!      .. Use Statements ..
       USE nag_library, ONLY : nag_wp
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       REAL (KIND=nag_wp), PARAMETER       ::                                  &
                                         f_target = -4.189828872724337E2_nag_wp
       REAL (KIND=nag_wp), PARAMETER       :: one = 1.0E0_nag_wp
       REAL (KIND=nag_wp), PARAMETER       :: zero = 0.0_nag_wp
       INTEGER, PARAMETER                  :: detail_level = 0,                &
                                              iuser_reference = 1,             &
                                              liopts = 100, lopts = 100,       &
                                              ndim = 20, nout = 6,             &
                                              report_freq = 100,               &
                                              ruser_global = 1
       REAL (KIND=nag_wp), PARAMETER       ::                                  &
                                   x_target(1:ndim) = -420.9687463599820_nag_wp
    CONTAINS
       SUBROUTINE objfun_schwefel_objective(ndim,x,f,w)
!         subroutine to calculate the objective using provided workspace

!         .. Use Statements ..
          USE nag_library, ONLY : ddot
!         .. Implicit None Statement ..
          IMPLICIT NONE
!         .. Scalar Arguments ..
          REAL (KIND=nag_wp), INTENT (OUT)    :: f
          INTEGER, INTENT (IN)                :: ndim
!         .. Array Arguments ..
          REAL (KIND=nag_wp), INTENT (OUT)    :: w(ndim)
          REAL (KIND=nag_wp), INTENT (IN)     :: x(ndim)
!         .. Intrinsic Functions ..
          INTRINSIC                              abs, sin, sqrt
!         .. Executable Statements ..
          w = abs(x)
          w = sqrt(w)
          w = sin(w)
          f = ddot(ndim,x,1,w,1)

          RETURN
       END SUBROUTINE objfun_schwefel_objective
       SUBROUTINE objfun_schwefel_gradient(ndim,x,g,w1,w2)
!         subroutine to calculate the gradient of the objective function with provided workspace

!         .. Use Statements ..
          USE nag_library, ONLY : dscal
!         .. Implicit None Statement ..
          IMPLICIT NONE
!         .. Scalar Arguments ..
          INTEGER, INTENT (IN)                :: ndim
!         .. Array Arguments ..
          REAL (KIND=nag_wp), INTENT (OUT)    :: g(ndim), w1(ndim), w2(ndim)
          REAL (KIND=nag_wp), INTENT (IN)     :: x(ndim)
!         .. Intrinsic Functions ..
          INTRINSIC                              abs, cos, sin, sqrt
!         .. Executable Statements ..
          CONTINUE

          w1 = sqrt(abs(x))
          w2 = cos(w1)
          g = sin(w1)
          CALL dscal(ndim,0.5E0_nag_wp,w1,1)
          g = g + w1*w2

          RETURN
       END SUBROUTINE objfun_schwefel_gradient
       SUBROUTINE objfun_schwefel(mode,ndim,x,objf,vecout,nstate,iuser,ruser)
!         Objfun routine for the Schwefel function for E05SAF.

!         .. Implicit None Statement ..
          IMPLICIT NONE
!         .. Scalar Arguments ..
          REAL (KIND=nag_wp), INTENT (INOUT)  :: objf
          INTEGER, INTENT (INOUT)             :: mode
          INTEGER, INTENT (IN)                :: ndim, nstate
!         .. Array Arguments ..
          REAL (KIND=nag_wp), INTENT (INOUT)  :: ruser(*), vecout(ndim)
          REAL (KIND=nag_wp), INTENT (IN)     :: x(ndim)
          INTEGER, INTENT (INOUT)             :: iuser(*)
!         .. Local Scalars ..
          INTEGER                             :: threadno, work_1, work_2
          LOGICAL                             :: evalobjf, evalobjg
!         .. External Functions ..
          INTEGER, EXTERNAL                   :: omp_get_thread_num
!         .. Intrinsic Functions ..
          INTRINSIC                              real
!         .. Executable Statements ..
!         Call OMP_GET_NUM_THREAD to get the current thread number.
!         The use of '!$' also allows this example to be compiled in serial
!         without openMP libraries.
          threadno = 0
!$        threadno = omp_get_thread_num()

!         Test NSTATE to indicate what stage of computation has been reached.
          SELECT CASE (nstate)
          CASE (2)

!            OBJFUN is called for the very first time.
!            This occurs in serial on thread 0 only.

!            Set up any constant global variables required in RUSER and IUSER
!            This may also be done prior to calling E05SAF
             ruser(ruser_global) = -f_target*real(ndim,nag_wp)

!            Set reference index for thread 0 workspace in IUSER
             iuser(iuser_reference+threadno) = ruser_global + 1

          CASE (3)
!            OBJFUN has been called for the first time on a new thread.

!            This occurs at the start of the main parallel region.
!            This will not occur on thread 0.

!            In this example we take the opportunity to set the reference index for this
!            threads unique section of RUSER workspace in IUSER.

!            Each thead requires 2*ndim real numbers.
             iuser(iuser_reference+threadno) = iuser(iuser_reference) + &
                2*ndim*threadno

          CASE (1)
!         OBJFUN is called on entry to a NAG local minimizer.
          CASE (0)
!         This will be the normal value of NSTATE.
          CASE DEFAULT
!            This is extremely unlikely, and indicates that an error has
!            occurred on the system
             mode = -1
             IF (detail_level>=2) THEN
                WRITE (nout,99999)
             END IF
             GO TO 20
          END SELECT

!         Test MODE to determine whether to calculate OBJF and/or OBJGRD.
          evalobjf = .FALSE.
          evalobjg = .FALSE.
          SELECT CASE (mode)
          CASE (0,5)
!            Only the value of the objective function is needed.
             evalobjf = .TRUE.
          CASE (1,6)
!            Only the values of the NDIM gradients are required.
             evalobjg = .TRUE.
          CASE (2,7)
!            Both the objective function and the NDIM gradients are required.
             evalobjf = .TRUE.
             evalobjg = .TRUE.
          CASE DEFAULT
             mode = -1
             WRITE (nout,99999)
             GO TO 20
          END SELECT

!         Set work_1 and work_2 to point to the section of RUSER reserved
!         for this thread
          work_1 = iuser(iuser_reference+threadno)
          work_2 = work_1 + ndim

          IF (evalobjf) THEN
!            Evaluate the objective function.
!            objf = const + sum( x*sin(abs(sqrt(x))))

!            Pass thread-safe workspace to objfun_schwefel_objective
             CALL objfun_schwefel_objective(ndim,x,objf,ruser(work_1))
!            Use constant safely available to all threads.
             objf = objf + ruser(ruser_global)

          END IF

          IF (evalobjg) THEN
!            Calculate the gradient of the objective function, 
!            and return the result in VECOUT.
!            gradient = sin(sqrt(abs(x))) 
!                       + sign(cos(sqrt(abs(x)))/2.0*sqrt(abs(x)),x)

             CALL objfun_schwefel_gradient(ndim,x,vecout,ruser(work_1), &
                ruser(work_2))
          END IF

20        CONTINUE
          RETURN

99999     FORMAT (1X,'** ERROR DETECTED ')

       END SUBROUTINE objfun_schwefel
       SUBROUTINE monmod(ndim,npar,x,xb,fb,xbest,fbest,itt,iuser,ruser,inform)
!         Example monitor function.

!         .. Implicit None Statement ..
          IMPLICIT NONE
!         .. Scalar Arguments ..
          REAL (KIND=nag_wp), INTENT (IN)     :: fb
          INTEGER, INTENT (INOUT)             :: inform
          INTEGER, INTENT (IN)                :: ndim, npar
!         .. Array Arguments ..
          REAL (KIND=nag_wp), INTENT (IN)     :: fbest(npar), xb(ndim),        &
                                                 xbest(ndim,npar)
          REAL (KIND=nag_wp), INTENT (INOUT)  :: ruser(*), x(ndim,npar)
          INTEGER, INTENT (IN)                :: itt(6)
          INTEGER, INTENT (INOUT)             :: iuser(*)
!         .. Local Scalars ..
          INTEGER                             :: k
!         .. Intrinsic Functions ..
          INTRINSIC                              modulo
!         .. Executable Statements ..
          IF (detail_level>=2) THEN
!            Report on the first iteration, and every report_freq iterations.
             IF (itt(1)==1 .OR. modulo(itt(1),report_freq)==0) THEN
                WRITE (nout,*)
                WRITE (nout,*) '* Current global optimum candidate:'
                DO k = 1, ndim
                   WRITE (nout,99999) k, xb(k)
                END DO
                WRITE (nout,*) '* Current global optimum value:'
                WRITE (nout,99998) fb
             END IF
          END IF

!         If required set INFORM<0 to force exit
          FLUSH (nout)

          RETURN
99999     FORMAT (1X,'* xb(',I3,') = ',F9.2)
99998     FORMAT (1X,'* fb = ',F9.5)

       END SUBROUTINE monmod
       SUBROUTINE display_result(ndim,xb,fb,itt,inform,ruser)
!         Display final results in comparison to known global optimum.

!         .. Use Statements ..
          USE nag_library, ONLY : x04cbf
!         .. Implicit None Statement ..
          IMPLICIT NONE
!         .. Parameters ..
          INTEGER, PARAMETER                  :: indent = 0, ncols = 80
          CHARACTER (10), PARAMETER           :: clabs(1:2) = (/               &
                                                 'x_target  ',  'xb        '  /)
          CHARACTER (1), PARAMETER            :: diag =   'N',  labcol =       &
                                                 'C',  labrow =   'I',         &
                                                 matrix =   'G'
          CHARACTER (4), PARAMETER            :: form = 'f9.2'
!         .. Scalar Arguments ..
          REAL (KIND=nag_wp), INTENT (IN)     :: fb
          INTEGER, INTENT (IN)                :: inform, ndim
!         .. Array Arguments ..
          REAL (KIND=nag_wp), INTENT (IN)     :: ruser(*), xb(ndim)
          INTEGER, INTENT (IN)                :: itt(6)
!         .. Local Scalars ..
          INTEGER                             :: ifail, ldxcom
          CHARACTER (ncols)                   :: title
!         .. Local Arrays ..
          REAL (KIND=nag_wp), ALLOCATABLE     :: xcom(:,:)
!         .. Executable Statements ..
!         Display final counters.
          WRITE (nout,99996) 'Total complete iterations             : ', &
             itt(1)
          WRITE (nout,99996) 'Complete iterations since improvement : ', &
             itt(2)
          WRITE (nout,99996) 'Total particles converged to xb       : ', &
             itt(3)
          WRITE (nout,99996) 'Total improvements to global optimum  : ', &
             itt(4)
          WRITE (nout,99996) 'Total function evaluations            : ', &
             itt(5)
          WRITE (nout,99996) 'Total particles re-initialized        : ', &
             itt(6)

!         Display why finalization occurred.
          WRITE (nout,*)
          SELECT CASE (inform)
          CASE (1)
             WRITE (nout,99995) 'Target value achieved'
          CASE (2)
             WRITE (nout,99995) 'Minimum swarm standard deviation obtained'
          CASE (3)
             WRITE (nout,99995) 'Sufficient particles converged'
          CASE (4)
             WRITE (nout,99995) 'No improvement in preset iteration limit'
          CASE (5)
             WRITE (nout,99995) 'Maximum complete iterations attained'
          CASE (6)
             WRITE (nout,99995) 'Maximum function evaluations exceeded'
          CASE (:-1)
             WRITE (nout,99994) inform
          CASE DEFAULT
          END SELECT

!         Display final objective value and location.
          WRITE (nout,*)
          WRITE (nout,99999) 0.0E0_nag_wp
          WRITE (nout,99998) fb
          FLUSH (nout)

          ldxcom = ndim
          ALLOCATE (xcom(ldxcom,2))
          xcom(1:ndim,1) = x_target(1:ndim)
          xcom(1:ndim,2) = xb(1:ndim)

          title = ' Comparison between known and achieved optima.'
          ifail = 0
          CALL x04cbf(matrix,diag,ndim,2,xcom,ldxcom,form,title,labrow,clabs, &
             labcol,clabs,ncols,indent,ifail)

          DEALLOCATE (xcom)

          WRITE (nout,99997)

          RETURN
99999     FORMAT ('  Known objective optimum  : ',F13.5)
99998     FORMAT ('  Achieved objective value : ',F13.5)
99997     FORMAT ('**--------------------**--------------------**'/)
99996     FORMAT (2X,A40,I16)
99995     FORMAT (2X,A43)
99994     FORMAT ('  User termination case                    :',I16)
       END SUBROUTINE display_result
    END MODULE e05safe_mod

    PROGRAM e05safe
!      E05SAF Example Main Program

!      This example program demonstrates how to use E05SAF in standard
!        execution, and with a selection of coupled local minimizers.
!      The non-default option 'REPEATABILITY ON' is used here, giving
!        repeatable results.

!      .. Use Statements ..
       USE nag_library, ONLY : e05saf, e05zkf
       USE e05safe_mod, ONLY : display_result, iuser_reference, liopts, lopts, &
                               monmod, nag_wp, ndim, nout, objfun_schwefel,    &
                               ruser_global
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Local Scalars ..
       REAL (KIND=nag_wp)                  :: fb
       INTEGER                             :: ifail, inform, liuser, lruser,   &
                                              max_threads, npar
       CHARACTER (80)                      :: optstr
!      .. Local Arrays ..
       REAL (KIND=nag_wp)                  :: bl(ndim), bu(ndim), opts(lopts), &
                                              xb(ndim)
       REAL (KIND=nag_wp), ALLOCATABLE     :: ruser(:)
       INTEGER                             :: iopts(liopts), itt(6)
       INTEGER, ALLOCATABLE                :: iuser(:)
!      .. External Functions ..
       INTEGER, EXTERNAL                   :: omp_get_max_threads
!      .. Intrinsic Functions ..
       INTRINSIC                              max
!      .. Executable Statements ..
!      Print advisory information.
       WRITE (nout,*) 'E05SAF Example Program Results'
       WRITE (nout,*)
       WRITE (nout,*) 'Minimization of the Schwefel function.'
       WRITE (nout,*)

!      Set problem specific values.
!      Set box bounds.       
       bl(1:ndim) = -500.0_nag_wp
       bu(1:ndim) = 500.0_nag_wp

!      Determine the number of threads to be used in the simulation
       max_threads = 1
!$     max_threads = omp_get_max_threads()

!      Ensure NPAR is sufficient for the number of threads
       npar = max(4000,5*max_threads)

!      Assign sufficient sizes for iuser and ruser.
!      In this example, RUSER will be split into workspace unique to each thread,
!      and IUSER will be used to store the indexes to the first element of 
!      workspace for each thread.

       liuser = max(30,iuser_reference+max_threads)
       lruser = ruser_global + 2*max_threads*ndim
       ALLOCATE (iuser(liuser),ruser(lruser))

       WRITE (nout,*) '* Solution without using coupled local minimizer *'

!      Call E05ZKF to initialize the option arrays
       ifail = 0
       CALL e05zkf('Initialize = E05SAF',iopts,liopts,opts,lopts,ifail)

!      Set the option SMP Callback Thread Safe to indicate the callback functions
!      are indeed threadsafe.  This must be done if using multiple threads.

       ifail = 0
       CALL e05zkf('SMP Callback Thread Safe = Yes',iopts,liopts,opts,lopts, &
          ifail)

!      Set various options to non-default values if required.
       ifail = 0
       CALL e05zkf('Repeatability = On',iopts,liopts,opts,lopts,ifail)
       ifail = 0
       CALL e05zkf('Verify Gradients = Off',iopts,liopts,opts,lopts,ifail)
       ifail = 0
       CALL e05zkf('Boundary = Hyperspherical',iopts,liopts,opts,lopts,ifail)
       ifail = 0
       CALL e05zkf('Maximum iterations static = 150',iopts,liopts,opts,lopts, &
          ifail)
       ifail = 0
       CALL e05zkf('Maximum function evaluations = 50000000',iopts,liopts, &
          opts,lopts,ifail)
       ifail = 0
       CALL e05zkf('Repulsion Initialize = 50',iopts,liopts,opts,lopts,ifail)
       ifail = 0
       CALL e05zkf('Repulsion Finalize = 30',iopts,liopts,opts,lopts,ifail)

!      Set an objective target.
       ifail = 0
       WRITE (optstr,99999) 'Target Objective Value', 1.0E0_nag_wp
       CALL e05zkf(optstr,iopts,liopts,opts,lopts,ifail)
       ifail = 0
       WRITE (optstr,99999) 'Target Objective Tolerance', 1.0E-5_nag_wp
       CALL e05zkf(optstr,iopts,liopts,opts,lopts,ifail)
       ifail = 0
       WRITE (optstr,99999) 'Target Objective Safeguard', 1.0E-2_nag_wp
       CALL e05zkf(optstr,iopts,liopts,opts,lopts,ifail)

!      Set some additional SMP library options if required.
       ifail = 0
       CALL e05zkf('SMP thread overrun = 100',iopts,liopts,opts,lopts,ifail)

       ifail = 0
       WRITE (optstr,99998) 'SMP Subswarm', max_threads
       CALL e05zkf(optstr,iopts,liopts,opts,lopts,ifail)

!      Call E05SAF to search for the global optimum.
!      As guaranteed success cannot by default be achieved, IFAIL=1,-1 is
!      recommended on entry.
       ifail = -1
       CALL e05saf(ndim,npar,xb,fb,bl,bu,objfun_schwefel,monmod,iopts,opts, &
          iuser,ruser,itt,inform,ifail)
!      It is essential to test IFAIL on exit.
       SELECT CASE (ifail)
       CASE (0,1)
!         E05SAF encountered no errors during operation,
!         and will have returned the best found optimum.
          CALL display_result(ndim,xb,fb,itt,inform,ruser)
       CASE (-999,-399,2:)
!         An error was detected.
          CONTINUE
       CASE DEFAULT
!         An instruction to exit was received by E05SAF from OBJFUN or MONMOD.
!         The exit flag will have been returned in INFORM.
          CALL display_result(ndim,xb,fb,itt,inform,ruser)
       END SELECT

!      ------------------------------------------------------------------
       WRITE (nout,*) '* Solution using coupled local minimizer E04JYF *'



!      Set the local minimizer to be E04JYF and set corresponding options.
       ifail = 0
       CALL e05zkf('Local Minimizer = E04JYF',iopts,liopts,opts,lopts,ifail)
       ifail = 0
       CALL e05zkf('Local Interior Iterations = 30',iopts,liopts,opts,lopts, &
          ifail)
       ifail = 0
       CALL e05zkf('Local Exterior Iterations = 200',iopts,liopts,opts,lopts, &
          ifail)
       ifail = 0
       WRITE (optstr,99999) 'Local Interior Tolerance', 1.0E-4_nag_wp
       CALL e05zkf(optstr,iopts,liopts,opts,lopts,ifail)
       ifail = 0
       WRITE (optstr,99999) 'Local Exterior Tolerance', 1.0E-8_nag_wp
       CALL e05zkf(optstr,iopts,liopts,opts,lopts,ifail)

!      Call E05SAF to search for the global optimum.
       ifail = -1
       CALL e05saf(ndim,npar,xb,fb,bl,bu,objfun_schwefel,monmod,iopts,opts, &
          iuser,ruser,itt,inform,ifail)

!      It is essential to test IFAIL on exit.
       SELECT CASE (ifail)
       CASE (0,1)
!         E05SAF encountered no errors during operation,
!         and will have returned the best found optimum.
          CALL display_result(ndim,xb,fb,itt,inform,ruser)
       CASE (-999,-399,2:)
!         An error was detected.
          CONTINUE
       CASE DEFAULT
!         An instruction to exit was received by E05SAF from OBJFUN or MONMOD.
!         The exit flag will have been returned in INFORM.
          CALL display_result(ndim,xb,fb,itt,inform,ruser)
       END SELECT

!      -----------------------------------------------------------------
       WRITE (nout,*) '* Solution using coupled local minimizer E04UCF *'

!      Set the local minimizer to be E04UCF and set corresponding options.
       ifail = 0
       CALL e05zkf('Local Minimizer = E04UCF',iopts,liopts,opts,lopts,ifail)
       ifail = 0
       CALL e05zkf('Local Interior Iterations = 10',iopts,liopts,opts,lopts, &
          ifail)
       ifail = 0
       CALL e05zkf('Local Exterior Iterations = 100',iopts,liopts,opts,lopts, &
          ifail)

!      Call E05SAF to search for the global optimum.
       ifail = -1
       CALL e05saf(ndim,npar,xb,fb,bl,bu,objfun_schwefel,monmod,iopts,opts, &
          iuser,ruser,itt,inform,ifail)

!      It is essential to test IFAIL on exit.
       SELECT CASE (ifail)
       CASE (0,1)
!         E05SAF encountered no errors during operation,
!         and will have returned the best found optimum.
          CALL display_result(ndim,xb,fb,itt,inform,ruser)
       CASE (-999,-399,2:)
!         An error was detected.
          CONTINUE
       CASE DEFAULT
!         An instruction to exit was received by E05SAF from OBJFUN or MONMOD.
!         The exit flag will have been returned in INFORM.
          CALL display_result(ndim,xb,fb,itt,inform,ruser)
       END SELECT

99999  FORMAT (A,' = ',E32.16)
99998  FORMAT (A,' = ',I20)
    END PROGRAM e05safe
