#!/bin/sh -u
#
# Installation script for NAG Library for SMP & Multicore Mark 23

get_installdir()
{
  installdir=""
  while [ "${installdir}" = "" ]; do
    echo "> "
    read ans
    if [ $ans ]; then
      case $ans in
      *) installdir=$ans ;;
      esac
    else
      installdir=${defaultinstalldir}
    fi
  done
  # Replace any ~ by ${HOME} otherwise you end up with a
  # subdirectory named ~ (dangerous if you then try to remove it!)
  installdir=`echo ${installdir} | sed -e "s%~%${HOME}%g"`
}

showLicense()
{
  licence_agreement_file=lic_agr.txt
  if [ -f ${licence_agreement_file} ]; then
    more ${licence_agreement_file}
  else
    echo "Cannot find licence agreement file ${licence_agreement_file}."
    echo "NAG Library not installed."
    exit
  fi

  reply=""
  while [ "${reply}" != "accept" -a "${reply}" != "decline" ]; do
    echo "[accept/decline]? :"
    read reply
    reply=`echo ${reply} | tr [:upper:] [:lower:]`
  done
}

umask 022

echo "               NAG Library Installation  "
echo "               ========================  "
echo ""
echo "This script installs the NAG Library for SMP & Multicore Mark 23."

toolname=`basename ${0}`

# Check for any switch arguments
doneswitches=0
badswitch=0
acceptlicense=0
docinstalldir=""
installdir=""
while test "${doneswitches}" = "0"
do
  case "${1-}" in
    -accept*)
      acceptlicense=1
      shift;;
    -docinstalldir=*)
    docinstalldir=`echo ${1} | sed -e 's%.*=%%'`
    shift;;
    -installdir=*)
    installdir=`echo ${1} | sed -e 's%.*=%%'`
    shift;;
    -*)
      echo "${toolname}: unrecognised switch: ${1}"
      badswitch=1
      exit
      shift;;
    *)
      doneswitches=1;;
  esac
done

# Default installation directory is under /opt if that directory already
# exists. Otherwise, default installation directory is under /usr/local.
if [ -d /opt ]; then
  defaultinstalldir=/opt/NAG
else
  defaultinstalldir=/usr/local/NAG
fi

if [ "${acceptlicense}" = "0" ]; then
  showLicense
  if [ "${reply}" != "accept" ]; then
    echo "Installation declined. NAG Library not installed."
    exit
  fi
fi

libtarfile=fs23.tar
if [ ! -f "${libtarfile}" ]; then
  echo "Installation file ${libtarfile} does not exist in the current"
  echo "directory. Installation failed."
  exit
fi

if [ "${installdir}" = "" ]; then
  echo ""
  echo "Where do you want to install the NAG Library for SMP & Multicore? "
  echo "Press return for default location (${defaultinstalldir})"
  echo "or enter an alternative path."
  echo "The directory will be created if it does not already exist."
  get_installdir
fi

echo ""
echo "Installing NAG Library for SMP & Multicore material in subdirectory"
echo "fsl6a23dfl of : ${installdir}"
echo ""

if [ -d ${installdir}/fsl6a23dfl ]; then
  echo "Subdirectory ${installdir}/fsl6a23dfl already exists."
  echo "Do you wish to overwrite it? (yes/no):"
  read ans
  ans=`echo ${ans} | tr [:upper:] [:lower:]`
  if [ "$ans" = "n" -o "$ans" = "no" ] ; then
     echo "Aborting installation"
     exit
  fi
fi

if [ ! -d ${installdir} ]; then
  mkdir -p ${installdir}
  if [ $? -ne 0 ]; then
    echo "***** Cannot create installation directory, installation failed *****"
    exit
  fi
fi

fromdir=`pwd`
echo "cd ${installdir}"
cd ${installdir}
fullpathinstalldir=`pwd`
echo "tar xf ${fromdir}/${libtarfile}"
tar xf ${fromdir}/${libtarfile}

echo ""
echo "NAG Library for SMP & Multicore material installed into directory"
echo "  ${installdir}/fsl6a23dfl"

# A list of all the nagsmp_example scripts that get installed
cd ${fullpathinstalldir}/fsl6a23dfl/scripts
examplescripts=`ls nagsmp_example*`
cd ${fullpathinstalldir}
# Edit the nagsmp_example scripts to point at the installed library directory
for script in ${examplescripts}
do
  if [ -f "${fullpathinstalldir}/fsl6a23dfl/scripts/${script}" ]; then
    echo ""
    echo "Editing the NAG example program script"
    echo "  ${fullpathinstalldir}/fsl6a23dfl/scripts/${script}"
    echo "to point at the NAG Library for SMP & Multicore installation directory ..."
    sed -e "s%nagfdir=.*%nagfdir=${fullpathinstalldir}/fsl6a23dfl%" < \
         ${fullpathinstalldir}/fsl6a23dfl/scripts/${script} > \
         ${fullpathinstalldir}/fsl6a23dfl/scripts/tmp.tmp
    if [ -f "${fullpathinstalldir}/fsl6a23dfl/scripts/tmp.tmp" ]; then
      mv ${fullpathinstalldir}/fsl6a23dfl/scripts/tmp.tmp \
         ${fullpathinstalldir}/fsl6a23dfl/scripts/${script}
      chmod 755 ${fullpathinstalldir}/fsl6a23dfl/scripts/${script}
    fi
  fi
done

cd ${fromdir}
# Now install Manual if it exists and is required
doctarfile=mk23_doc.tar
if [ ! -f "${doctarfile}" ]; then
  echo ""
  echo "Documentation file ${doctarfile} does not exist in the current"
  echo "directory. NAG Mark 23 Library Manual will not be installed."
else

  install_doc=0
  echo
  echo "Do you wish to install NAG Mark 23 Library Manual? (yes/no):"
  read ans
  ans=`echo ${ans} | tr [:upper:] [:lower:]`

  if [ "$ans" = "y" -o "$ans" = "ye" -o "$ans" = "yes" ] ; then
    defaultinstalldir=${installdir}
    echo ""
    echo "Where do you want to install the NAG Mark 23 Library Manual?"
    echo "Press return for default location (${defaultinstalldir})"
    echo "or enter an alternative path."
    echo "The directory will be created if it does not already exist."

    get_installdir

    echo ""
    echo "Installing NAG Mark 23 Library Manual in subdirectory"
    echo "nagdoc_fl23 of : ${installdir}"
    echo ""

    if [ -d ${installdir}/nagdoc_fl23 ]; then
      echo "Subdirectory ${installdir}/nagdoc_fl23 already exists."
      echo "Do you wish to overwrite it? (yes/no):"
      read ans
      ans=`echo ${ans} | tr [:upper:] [:lower:]`
      if [ "$ans" = "n" -o "$ans" = "no" ] ; then
         echo "Aborting installation"
         exit
      fi
    fi

    if [ ! -d "${installdir}" ]; then
      mkdir -p "${installdir}"
      if [ $? -ne 0 ]; then
        echo "***** Cannot create installation directory, installation failed *****"
        exit
      fi
    fi

    echo "cd ${installdir}"
    cd "${installdir}"
    echo "tar xf ${fromdir}/${doctarfile}"
    tar xf ${fromdir}/${doctarfile}

    echo ""
    echo "NAG Mark 23 Library Manual installed into directory"
    echo "${installdir}/nagdoc_fl23"
    echo ""
  else
    echo "NAG Mark 23 Library Manual not installed."
  fi
fi

# Determine whether there is more than one variant of the installed
# lib directory, and if so issue a message telling the installer to
# read the Installer's Note to find out what to do.
nlibdirs=`ls -ld ${fullpathinstalldir}/fsl6a23dfl/lib* | wc -l`
if [ ${nlibdirs} -gt 1 ]; then
  echo "Note that this installation contains more than one variant of the"
  echo "NAG \"lib\" directory:"
  ls -ld ${fullpathinstalldir}/fsl6a23dfl/lib*
  echo "See the Installer's Note in.html for instructions on how to rename one"
  echo "of these directories to lib."
fi

echo "====== NAG Library for SMP & Multicore installation complete ======"
